<?php

declare(strict_types=1);

namespace Drupal\se_bill;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Access controller for the Bill entity.
 *
 * @see \Drupal\se_bill\Entity\Bill.
 */
class BillAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\se_bill\Entity\BillInterface $entity */

    switch ($operation) {
      case 'view':
        if (!$entity->isPublished()) {
          return AccessResult::allowedIfHasPermission($account, 'view changes to bill entities');
        }
        return AccessResult::allowedIfHasPermission($account, 'view bill entities');

      case 'update':
        return AccessResult::allowedIfHasPermission($account, 'edit bill entities');

      case 'delete':
        return AccessResult::allowedIfHasPermission($account, 'delete bill entities');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add bill entities');
  }

}
