<?php

declare(strict_types=1);

namespace Drupal\se_payment;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Access controller for the Payment entity.
 *
 * @see \Drupal\se_payment\Entity\Payment.
 */
class PaymentAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\se_payment\Entity\PaymentInterface $entity */

    switch ($operation) {
      case 'view':
        if (!$entity->isPublished()) {
          return AccessResult::allowedIfHasPermission($account, 'view changes to payment entities');
        }
        return AccessResult::allowedIfHasPermission($account, 'view payment entities');

      case 'update':
        return AccessResult::allowedIfHasPermission($account, 'edit payment entities');

      case 'delete':
        return AccessResult::allowedIfHasPermission($account, 'delete payment entities');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add payment entities');
  }

}
